/* $Id: uc_utils.c,v 1.2 2018/06/01 09:16:14 rlr Exp $ */

/*
 * Copyright (c) 2018 Ross L Richardson <rueda@rlr.id.au>
 *
 * Permission to use, copy, modify, and distribute this software for any
 * purpose with or without fee is hereby granted, provided that the above
 * copyright notice and this permission notice appear in all copies.
 *
 * THE SOFTWARE IS PROVIDED "AS IS" AND THE AUTHOR DISCLAIMS ALL WARRANTIES
 * WITH REGARD TO THIS SOFTWARE INCLUDING ALL IMPLIED WARRANTIES OF
 * MERCHANTABILITY AND FITNESS. IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR
 * ANY SPECIAL, DIRECT, INDIRECT, OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES
 * WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS, WHETHER IN AN
 * ACTION OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT OF
 * OR IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
 */


#include "uc_utils.h"


bool
uc_is_space(uint32_t cp)
{
	return cp == 0x0020U || cp == 0x00A0U || cp == 0x1680U || cp == 0x180EU
	    || (0x2000U <= cp && cp <= 0x200BU) || cp == 0x202FU
	    || cp == 0x205FU || cp == 0x3000U || cp == 0xFEFFU;
}


bool
uc_is_eol(uint32_t cp)
{
	return (0x000AU <= cp && cp <= 0x000DU) || cp == 0x0085U
	    || cp == 0x2028U || cp == 0x2029U;
}


bool
uc_is_mes2(uint32_t cp)
{
	uint32_t	 bl;
	bool	 m;

	bl = cp & 0x000000FFU;

	switch (cp >> 8) {

	case 0x00000000U :
		m = (0x0020U <= bl && bl <= 0x007EU)
		    || (0x00A0U <= bl && bl <= 0x00FFU);
	break;

	case 0x00000001U :
		m = bl <= 0x007FU || bl == 0x008FU || bl == 0x0092U
		    || bl == 0x00B7U || (0x00DEU <= bl && bl <= 0x00EFU)
		    || (0x00FAU <= bl && bl <= 0x00FFU);
	break;

	case 0x00000002U :
		m = (0x0018U <= bl && bl <= 0x001BU)
		    || 0x001EU == bl || bl == 0x001FU
		    || bl == 0x0059U || bl == 0x007CU || bl == 0x0092U
		    || (0x00BBU <= bl && bl <= 0x00BDU)
		    || bl == 0x00C6U || bl == 0x00C7U || bl == 0x00C9U
		    || bl == 0x00D6U || (0x00D8U <= bl && bl <= 0x00DDU)
		    || bl == 0x00DFU || bl == 0x00EEU;
	break;

	case 0x00000003U :
		m = bl == 0x0074U || bl == 0x0075U || bl == 0x007AU
		    || bl == 0x007EU || (0x0084U <= bl && bl <= 0x008AU)
		    || bl == 0x008CU || (0x008EU <= bl && bl <= 0x00A1U)
		    || (0x00A3U <= bl && bl <= 0x00CEU) || bl == 0x00D7U
		    || (0x00DAU <= bl && bl <= 0x00E1U);
	break;

	case 0x00000004U :
		m = bl <= 0x005FU || (0x0090U <= bl && bl <= 0x00C4U)
		    || bl == 0x00C7U || bl == 0x00C8U
		    || bl == 0x00CBU || bl == 0x00CCU
		    || (0x00D0U <= bl && bl <= 0x00EBU)
		    || (0x00EEU <= bl && bl <= 0x00F5U)
		    || bl == 0x00F8U || bl == 0x00F9U;
	break;

	case 0x0000001EU :
		m = bl == 0x0002U || bl == 0x0003U || bl == 0x000AU
		    || bl == 0x000BU || bl == 0x001EU || bl == 0x001FU
		    || bl == 0x0040U || bl == 0x0041U || bl == 0x0056U
		    || bl == 0x0057U || bl == 0x0060U || bl == 0x0061U
		    || bl == 0x006AU || bl == 0x006BU
		    || (0x0080U <= bl && bl <= 0x0085U)
		    || bl == 0x009BU || bl == 0x00F2U || bl == 0x00F3U;
	break;

	case 0x0000001FU :
		m = bl <= 0x0015U || (0x0018U <= bl && bl <= 0x001DU)
		    || (0x0020U <= bl && bl <= 0x0045U)
		    || (0x0048U <= bl && bl <= 0x004DU)
		    || (0x0050U <= bl && bl <= 0x0057U)
		    || bl == 0x0059U || bl == 0x005BU || bl == 0x005DU
		    || (0x005FU <= bl && bl <= 0x007DU)
		    || (0x0080U <= bl && bl <= 0x00B4U)
		    || (0x00B6U <= bl && bl <= 0x00C4U)
		    || (0x00C6U <= bl && bl <= 0x00D3U)
		    || (0x00D6U <= bl && bl <= 0x00DBU)
		    || (0x00DDU <= bl && bl <= 0x00EFU)
		    || (0x00F2U <= bl && bl <= 0x00F4U)
		    || (0x00F6U <= bl && bl <= 0x00FEU);
	break;

	case 0x00000020U :
		m = (0x0013U <= bl && bl <= 0x0015U)
		    || (0x0017U <= bl && bl <= 0x001EU)
		    || (0x0020U <= bl && bl <= 0x0022U)
		    || bl == 0x0026U || bl == 0x0030U || bl == 0x0032U
		    || bl == 0x0033U || bl == 0x0039U || bl == 0x003AU
		    || bl == 0x003CU || bl == 0x003EU || bl == 0x0044U
		    || bl == 0x004AU || bl == 0x007FU || bl == 0x0082U
		    || bl == 0x00A3U || bl == 0x00A4U || bl == 0x00A7U
		    || bl == 0x00ACU || bl == 0x00AFU;
	break;

	case 0x00000021U :
		m = bl == 0x0005U || bl == 0x0013U || bl == 0x0016U
		    || bl == 0x0022U || bl == 0x0026U || bl == 0x002EU
		    || (0x005BU <= bl && bl <= 0x005EU)
		    || (0x0090U <= bl && bl <= 0x0095U) || bl == 0x00A8U;
	break;

	case 0x00000022U :
		m = bl == 0x0000U || bl == 0x0002U || bl == 0x0003U
		    || bl == 0x0006U || bl == 0x0008U || bl == 0x0009U
		    || bl == 0x000FU || bl == 0x0011U || bl == 0x0012U
		    || bl == 0x0015U || bl == 0x0019U || bl == 0x001AU
		    || bl == 0x001EU || bl == 0x001FU
		    || (0x0027U <= bl && bl <= 0x002BU) || bl == 0x0048U
		    || bl == 0x0059U || bl == 0x0060U || bl == 0x0061U
		    || bl == 0x0064U || bl == 0x0065U || bl == 0x0082U
		    || bl == 0x0083U || bl == 0x0095U || bl == 0x0097U;
	break;

	case 0x00000023U :
		m = bl == 0x0002U || bl == 0x000AU || bl == 0x0020U
		    || bl == 0x0021U || bl == 0x0029U || bl == 0x002AU;
	break;

	case 0x00000025U :
		m = bl == 0x0000U || bl == 0x0002U || bl == 0x000CU
		    || bl == 0x0010U || bl == 0x0014U || bl == 0x0018U
		    || bl == 0x001CU || bl == 0x0024U || bl == 0x002CU
		    || bl == 0x0034U || bl == 0x003CU
		    || (0x0050U <= bl && bl <= 0x006CU) || bl == 0x0080U
		    || bl == 0x0084U || bl == 0x0088U || bl == 0x008CU
		    || (0x0090U <= bl && bl <= 0x0093U) || bl == 0x00A0U
		    || bl == 0x00A1U || (0x00AAU <= bl && bl <= 0x00ACU)
		    || bl == 0x00B2U || bl == 0x00BAU || bl == 0x00BCU
		    || bl == 0x00C4U || bl == 0x00CAU || bl == 0x00CBU
		    || bl == 0x00CFU || bl == 0x00D8U || bl == 0x00D9U
		    || bl == 0x00E6U;
	break;

	case 0x00000026U :
		m = (0x003AU <= bl && bl <= 0x003CU) || bl == 0x0040U
		    || bl == 0x0042U || bl == 0x0060U || bl == 0x0063U
		    || bl == 0x0065U || bl == 0x0066U || bl == 0x006AU
		    || bl == 0x006BU;
	break;

	case 0x000000F0U :
		m = bl == 0x0001U || bl == 0x0002U;
	break;

	case 0x000000FBU :
		m = bl == 0x0001U || bl == 0x0002U;
	break;

	case 0x000000FFU :
		m = bl == 0x00FDU;
	break;

	default :
		m = false;
	break;

	}

	return m;
}

