/* $Id: uc_utils.h,v 1.7 2018/06/19 04:58:01 rlr Exp $ */

/*
 * Copyright (c) 2018 Ross L Richardson <rueda@rlr.id.au>
 *
 * Permission to use, copy, modify, and distribute this software for any
 * purpose with or without fee is hereby granted, provided that the above
 * copyright notice and this permission notice appear in all copies.
 *
 * THE SOFTWARE IS PROVIDED "AS IS" AND THE AUTHOR DISCLAIMS ALL WARRANTIES
 * WITH REGARD TO THIS SOFTWARE INCLUDING ALL IMPLIED WARRANTIES OF
 * MERCHANTABILITY AND FITNESS. IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR
 * ANY SPECIAL, DIRECT, INDIRECT, OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES
 * WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS, WHETHER IN AN
 * ACTION OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT OF
 * OR IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
 */


/*
 * Module providing Unicode code point tests.
 */


#ifndef _UC_UTILS_H_
#define _UC_UTILS_H_


#include <sys/types.h>

#include <stdbool.h>


/* Codepoint limits */
#define UC_CP_MAX              0x10FFFFU
#define UC_CP_BASIC_LATIN_MIN  0x000020U
#define UC_CP_BASIC_LATIN_MAX  0x00007EU
#define UC_CP_MES2_MIN         UC_CP_BASIC_LATIN_MIN
#define UC_CP_MES2_MAX         0x00FFFDU


/*
 * bool
 * uc_is_space(uint32_t cp);
 *
 * Returns true if and only if cp is a Unicode space code point.
 */
bool	 uc_is_space(uint32_t);


/*
 * bool
 * uc_is_eol(uint32_t cp);
 *
 * Returns true if and only if cp is a Unicode end-of-line code point.
 */
bool	 uc_is_eol(uint32_t);


/*
 * bool
 * uc_is_mes2(uint32_t cp);
 *
 * Returns true if and only if cp is a member of the MES-2 subset of Unicode.
 */
bool	 uc_is_mes2(uint32_t);


#endif /* !_UC_UTILS_H_ */

